//-----------------------------------------------------------------------------
// EXT_MOD_LCD_2x16
// Version 1.4
// 02.11.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer ein externes Modul
// Nur fuer das Modul : myAVR LCD Add-On (von myAVR.de)
// 2 Zeilen x 16 Zeichen LCD-Text-Display
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/"
// liegen und mit #include <UB_EXT_MODULE/EXT_MOD_LCD_2x16.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : Port-J bei MK3 (komplett) (bzw. Port-D bei MK1/MK2/mySmartControl)
//-----------------------------------------------------------------------------
//
// Hinweis : Jumper "R/W" muss auf "Aus" gesetzt sein
//           Jumper "Beleuchtung" muss auf "AN" gesetzt sein
//
// > der DEFINE-Schalter "TLCD_PIN_MODE" muss zur Hardware passen 
//
//-----------------------------------------------------------------------------

#include <stdio.h>



//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "EXT_INIT_TLCD" starten
// 02. Funktion "MOD_TLCD_CLEAR" loescht das Display
// 03. Funktion "MOD_TLCD_MODE" aendert den Anzeigemode (AUS,EIN,CURSOR,BLINK)
// 04. Funktion "MOD_TLCD_STRING_AT" zeigt einen String an Pos (x,y) an
// 05. Funktion "MOD_TLCD_UINT_AT" zeigt eine Zahl an Pos (x,y) an
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------

typedef enum {
  TDISP_OFF =0,    // Display=AUS, Cursor=Aus, Blinken=Aus
  TDISP_ON,        // Display=EIN, Cursor=Aus, Blinken=Aus
  TDISP_CURSOR,    // Display=EIN, Cursor=EIN, Blinken=Aus
  TDISP_BLINK      // Display=EIN, Cursor=EIN, Blinken=EIN
}TLCD_Mode;



//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void EXT_INIT_TLCD(void);
void MOD_TLCD_CLEAR(void);
void MOD_TLCD_MODE(TLCD_Mode mode);
void MOD_TLCD_STRING_AT(char xp, char yp, const char *ptr);
void MOD_TLCD_UINT_AT(char xp, char yp, unsigned int wert);
void MOD_TLCD_SET_CURSOR(char xp,char yp);


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
void tlcd_sendbyte(char wert,char mode);
void tlcd_command(char wert);
void tlcd_data(char wert);
void ub_pause_TLCD(unsigned int p); 
void tlcd_clock(void);


//-----------------------------------------------------------------------------
// Hardware auswahl
//-----------------------------------------------------------------------------
#define TLCD_PIN_MODE 0         // 0=MK3-Board, 1=MK1/MK2-Board/mySmartControl

#if TLCD_PIN_MODE==0
  // Hardware = myAVR MK3-Board
  #define MK3_DDR_TLCD DDRJ	  // DDR-Register vom Text-LCD
  #define MK3_PORT_TLCD PORTJ	  // Port vom Text-LCD

  #define TLCD_RS  0x01           // Bit0 = (RS)      Pin-4 am Display
  #define TLCD_E   0x02           // Bit1 = (E)       Pin-6 am Display
  #define TLCD_DB4 0x04           // Bit2 = (Data D4) Pin-11 am Display
  #define TLCD_DB5 0x08           // Bit3 = (Data D5) Pin-12 am Display
  #define TLCD_DB6 0x10           // Bit4 = (Data D6) Pin-13 am Display
  #define TLCD_DB7 0x20           // Bit5 = (Data D7) Pin-14 am Display
#else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl
  #define MK3_DDR_TLCD DDRD       // DDR-Register vom Text-LCD 
  #define MK3_PORT_TLCD PORTD     // Port vom Text-LCD 

  #define TLCD_RS  0x04           // Bit2 = (RS)      Pin-4 am Display
  #define TLCD_E   0x08           // Bit3 = (E)       Pin-6 am Display
  #define TLCD_DB4 0x10           // Bit4 = (Data D4) Pin-11 am Display
  #define TLCD_DB5 0x20           // Bit5 = (Data D5) Pin-12 am Display
  #define TLCD_DB6 0x40           // Bit6 = (Data D6) Pin-13 am Display
  #define TLCD_DB7 0x80           // Bit7 = (Data D7) Pin-14 am Display
#endif

//-----------------------------------------------------------------------------
// Display Groesse
//-----------------------------------------------------------------------------
#define TLCD_MAX_XPOS  16       // Zeichen (0..15)
#define TLCD_MAX_YPOS  2        // Zeilen (0..1)

//-----------------------------------------------------------------------------
// Display kommandos (siehe Datenblatt)
//-----------------------------------------------------------------------------
#define  TLCD_CMD_CLEAR          0x01   // loescht das Display
#define  TLCD_CMD_ENTRY_MODE     0x06   // Cursor increase, Display fix
#define  TLCD_CMD_DISP_M0        0x08   // Display=AUS, Cursor=Aus, Blinken=Aus
#define  TLCD_CMD_DISP_M1        0x0C   // Display=EIN, Cursor=AUS, Blinken=Aus
#define  TLCD_CMD_DISP_M2        0x0E   // Display=EIN, Cursor=EIN, Blinken=Aus
#define  TLCD_CMD_DISP_M3        0x0F   // Display=EIN, Cursor=EIN, Blinken=EIN
#define  TLCD_CMD_INIT_DISPLAY   0x28   // 2 Zeilen Display, 5x7 Punkte


//-----------------------------------------------------------------------------
// Die Pausenzeiten sind fuer 16MHz angepasst
//-----------------------------------------------------------------------------
#define  TLCD_pause1  1000           // pause1 beim init (als Schleifen_Endwert)
#define  TLCD_pause2  100            // pause2 beim init (als Schleifen_Endwert)
#define  TLCD_pause3  1000           // pause3 beim clear (als Schleifen_Endwert)
#define  TLCD_pause4  100            // pause4 beim senden (als Schleifen_Endwert)


//-----------------------------------------------------------------------------
void EXT_INIT_TLCD(void) {	// zum initialisieren der Text-Displays
  // als Ausgang
  MK3_DDR_TLCD = 0xFF;
  MK3_PORT_TLCD = 0x00; // alle Pins auf Lo

  ub_pause_TLCD(TLCD_pause1);

  // Init-Sequenz
  MK3_PORT_TLCD|=TLCD_DB4;  
  MK3_PORT_TLCD|=TLCD_DB5;

  // 1. Init-Impuls
  tlcd_clock();
  ub_pause_TLCD(TLCD_pause2);

  // 2. Init-Impuls
  tlcd_clock();
  ub_pause_TLCD(TLCD_pause2);

  // 3. Init-Impuls
  tlcd_clock();
  ub_pause_TLCD(TLCD_pause2);

  // LCD-Modus einstellen
  MK3_PORT_TLCD&=~TLCD_DB4;  // 4Bit-Interface
  MK3_PORT_TLCD|=TLCD_DB5;
  tlcd_clock();
  ub_pause_TLCD(TLCD_pause2);

  // LCD-Settings einstellen
  tlcd_command(TLCD_CMD_INIT_DISPLAY);
  tlcd_command(TLCD_CMD_ENTRY_MODE);

  // Display einschalten
  MOD_TLCD_MODE(TDISP_ON); // Display EIN, Cursor AUS, Blinken AUS

  // Display loeschen
  MOD_TLCD_CLEAR();  
}


//-----------------------------------------------------------------------------
void MOD_TLCD_CLEAR(void) {
  // loescht das Display
  
  tlcd_command(TLCD_CMD_CLEAR);
  ub_pause_TLCD(TLCD_pause3);
}


//-----------------------------------------------------------------------------
void MOD_TLCD_MODE(TLCD_Mode mode) {
  // stellt einen Display-Mode ein

  char wert=0x00;

  if(mode==TDISP_OFF) wert=TLCD_CMD_DISP_M0;
  if(mode==TDISP_ON) wert=TLCD_CMD_DISP_M1;
  if(mode==TDISP_CURSOR) wert=TLCD_CMD_DISP_M2;
  if(mode==TDISP_BLINK) wert=TLCD_CMD_DISP_M3;

  tlcd_command(wert);
}


//-----------------------------------------------------------------------------
void MOD_TLCD_STRING_AT(char xp, char yp, const char *ptr) {
  // schreibt einen String aufs Display
  // an Position x,y

  char c;
  unsigned int n;

  // Cursor positionieren
  MOD_TLCD_SET_CURSOR(xp,yp);

  // kompletter String anzeigen
  for(n=0;n<TLCD_MAX_XPOS;n++) {
    c=ptr[n];
    if(c==0) {
      break; 
    }
    else {
      tlcd_data(c);
    }   
  }
}

//-----------------------------------------------------------------------------
void MOD_TLCD_UINT_AT(char xp, char yp, unsigned int wert) {
  // schreibt eine U-INT Zahl aufs Display
  // an Position x,y

  char buf[7];

  // zahl in string wandeln

  sprintf(buf,"%5u",wert); // 5 stellige unsigned_int (ohne fuehrenden Nullen)
//  sprintf(buf,"%05u",wert); // 5 stellige unsigned_int (mit fuehrenden Nullen)

  // zahl ausgeben
  MOD_TLCD_STRING_AT(xp,yp,buf);

}


//-----------------------------------------------------------------------------
void MOD_TLCD_SET_CURSOR(char xp,char yp) {
  // setzt den Cursor an die
  // X,Y Position

  char wert=0x00;

  if(xp>=TLCD_MAX_XPOS) xp=0;
  if(yp>=TLCD_MAX_YPOS) yp=0;

  yp=(yp<<6);     // yp=bit6
  wert=(xp|yp);   // xp=Bit5 bis Bit0
  wert|=0x80;     // Bit7 setzen
  
  tlcd_command(wert);
}


//-----------------------------------------------------------------------------
void tlcd_sendbyte(char wert,char mode) { // INTERNE-FUNKTION
  // sendet byte
  // im 4bit Mode

  char dummy=0x00;

#if TLCD_PIN_MODE==0
  if(mode==0) {
    // command-byte
    dummy=(wert&0xF0);  // Hi-Nibble
    dummy=(dummy>>2);
    dummy=(dummy&0x3C);
    dummy&=~TLCD_RS;    // RS auf Lo = Command
    MK3_PORT_TLCD=dummy;
    tlcd_clock();
    dummy=(wert&0x0F);  // Lo-Nibble
    dummy=(dummy<<2);
    dummy&=~TLCD_RS;    // RS auf Lo = Command
    MK3_PORT_TLCD=dummy;
    tlcd_clock();
    ub_pause_TLCD(TLCD_pause4);
  }
  else {
    // data-byte
    dummy=(wert&0xF0);  // Hi-Nibble
    dummy=(dummy>>2);
    dummy=(dummy&0x3C);
    dummy|=TLCD_RS;     // RS auf Hi = Data
    MK3_PORT_TLCD=dummy;
    tlcd_clock();
    dummy=(wert&0x0F);  // Lo-Nibble
    dummy=(dummy<<2);
    dummy|=TLCD_RS;     // RS auf Hi = Data
    MK3_PORT_TLCD=dummy;
    tlcd_clock();
    ub_pause_TLCD(TLCD_pause4);
  }
#else
  if(mode==0) {
    // command-byte
    dummy=(wert&0xF0);  // Hi-Nibble
    dummy&=~TLCD_RS;    // RS auf Lo = Command
    MK3_PORT_TLCD=dummy;
    tlcd_clock();
    dummy=(wert&0x0F);  // Lo-Nibble
    dummy=(dummy<<4);
    dummy&=~TLCD_RS;    // RS auf Lo = Command
    MK3_PORT_TLCD=dummy;
    tlcd_clock();
    ub_pause_TLCD(TLCD_pause4);
  }
  else {
    // data-byte
    dummy=(wert&0xF0);  // Hi-Nibble
    dummy|=TLCD_RS;     // RS auf Hi = Data
    MK3_PORT_TLCD=dummy;
    tlcd_clock();
    dummy=(wert&0x0F);  // Lo-Nibble
    dummy=(dummy<<4);
    dummy|=TLCD_RS;     // RS auf Hi = Data
    MK3_PORT_TLCD=dummy;
    tlcd_clock();
    ub_pause_TLCD(TLCD_pause4);
  }
#endif

}


//-----------------------------------------------------------------------------
void tlcd_command(char wert) { // INTERNE-FUNKTION
  // sendet Command-Byte
  // ans Display

  tlcd_sendbyte(wert,0);  // RS auf Lo = Command

}

//-----------------------------------------------------------------------------
void tlcd_data(char wert) { // INTERNE-FUNKTION
  // sendet Data-Byte
  // ans Display

  tlcd_sendbyte(wert,1);  // RS auf Hi = Data

}



//-----------------------------------------------------------------------------
void tlcd_clock(void) { // INTERNE-FUNKTION
  // erzeugt einen Clock-Impuls
  
  MK3_PORT_TLCD|=TLCD_E;  // auf Hi
  MK3_PORT_TLCD&=~TLCD_E; // auf Lo
}


//-----------------------------------------------------------------------------
void ub_pause_TLCD(unsigned int p) { // kleine pause INTERNE-FUNKTION
  unsigned int n=0;

  for(n=0;n<=p;n++) {
    // kleine Pause
    __asm volatile ("nop \n");  // mini Pause
  }
}







