//-----------------------------------------------------------------------------
// MK3_2560_ADC
// Version 1.4
// 29.05.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board (oder MK2 Board)
// mit ATMega-2560 @ 16MHz (oder ATMega8)
// ADC-Eingabe
// ADC an PortF Bit0 bis Bit7 (bzw. PortC0-C7)
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_ADC.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : @MK3 (ATMega2560) Port-F (komplett)
// Belegt : @MK2 (ATMega8)    Port-C (komplett)
//
// Als Voltage Referenz wird per DEFAULT der Vcc-Pin (5V) benutzt
// (kann per Software auf AREF-Pin, oder  interne-Ref umgeschaltet werden)
//
// Hinweis :
//
// > der DEFINE-Schalter "ADC_HW_MODE"
//   muss zur Hardware passen !!
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------

#define USE_MK3_ADC_READ_CH 1
#define USE_MK3_ADC_MITTEL_CH 1
#define USE_MK3_ADC_REF 1

//-----------------------------------------------------------------------------
// Hardware auswahl
//-----------------------------------------------------------------------------
#define ADC_HW_MODE    0       // 0=MK3-Board, 1=MK1/MK2-Board/mySmartControl


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_ADC" starten
// 02. Funktion "MK3_ADC_CHANNEL" stellt einen ADC-Kanal ein
// 03. Funktion "MK3_ADC_READ" liest einen AD-Wert vom eingestellten Kanal
// 04. Funtkion "MK3_ADC_READ_CH" liest einen bestimten Kanal aus
// 05. Funktion "MK3_ADC_MITTEL_CH" bildet den Mittelwert aus einem Kanal
// 06. Funktion "MK3_ADC_SET_REF" stellt die Referenz-Spannung ein
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------

typedef enum {
  ADC0 = 0,
  ADC1,
  ADC2,
  ADC3,
  ADC4,
  ADC5,
  ADC6,
  ADC7
}ADC_KAN;

typedef enum {
  MW_1 = 0,  // macht eine Messung
  MW_2,      // macht zwei Messungen und bildet den Mittelwert
  MW_4,      // macht vier Messungen und bildet den Mittelwert
  MW_8       // macht acht Messungen ...
}ADC_MITTEL;

typedef enum {
  REF_AREF = 0, // Referenz = AREF-Pin
  REF_AVCC,     // Referenz = AVCC-Pin (5V)
  REF_INT       // Referenz = Interne Ref (2,56V)
}ADC_REF;



//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_ADC(void);	 // zum initialisieren des AD-Wandlers
void MK3_ADC_CHANNEL(ADC_KAN kan); // zum einstellen des Kanales
unsigned int MK3_ADC_READ(void); // zum lesen des ADC (wenn Kanal schon gewaehlt)
unsigned int MK3_ADC_READ_CH(ADC_KAN kan); // zum lesen eines Kanales
unsigned int MK3_ADC_MITTEL_CH(ADC_KAN kan,ADC_MITTEL mw); // Mittelwertbildung
void MK3_ADC_SET_REF(ADC_REF uref); // umschalten der Referenz-Spannung




//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------
#if ADC_HW_MODE==0
  // Hardware = myAVR MK3-Board 
  #define MK3_DDR_ADC DDRF   // DDR-Register vom ADC
#else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl
  #define MK3_DDR_ADC DDRC   // DDR-Register vom ADC
#endif


//-----------------------------------------------------------------------------
void MK3_INIT_ADC(void) {	// zum initialisieren des AD-Wandlers
  // alle ADC initialisieren
  // als Eingang
  MK3_DDR_ADC = 0x00;

#if ADC_HW_MODE==0
  // Hardware = myAVR MK3-Board 

  // Register (ADMUX)
  ADMUX=0x40;			// REF=Vcc-Pin

  // Register (ADCSRB)
  ADCSRB=0x00;			// FREE-Running Mode

  // Register (ADCSRA)
  ADCSRA=0x84;			// ADC-Enable, Clk-Frq=1MHz (grenze=1MHz)

  // Register (DIDR0)
  DIDR0=0x00;

  // Register (DIDR2)
  DIDR2=0x00;

#else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl

  // Register (ADMUX)
  ADMUX=0x40;			// REF=Vcc-Pin

  // Register (ADCSRA)
  ADCSRA=0xA4;			// ADC-Enable, FREE-Running Mode, Clk-Frq=1MHz (grenze=1MHz)
#endif

}

//-----------------------------------------------------------------------------
void MK3_ADC_CHANNEL(ADC_KAN kan) { // zum einstellen des Kanales
  char wert=0;

  if(kan==ADC0) wert=0;
  if(kan==ADC1) wert=1;
  if(kan==ADC2) wert=2;
  if(kan==ADC3) wert=3;
  if(kan==ADC4) wert=4;
  if(kan==ADC5) wert=5;
  if(kan==ADC6) wert=6;
  if(kan==ADC7) wert=7;

  ADMUX=ADMUX&0xE0;
  ADMUX=ADMUX|wert;

  // dummy read machen
  // weil der erste Wert nach dem umschalten
  // nicht stimmt
  MK3_ADC_READ();
}

//-----------------------------------------------------------------------------
unsigned int MK3_ADC_READ(void) { // zum lesen des ADC (Kanal muss eingestellt sein)
  unsigned int ret_wert=0;
  unsigned char wert_lo,wert_hi;

  ADCSRA|=0x40;	// starte convertierung

  while((ADCSRA&0x10)==0) {
    // warte bis messung fertig
  }

  wert_lo=ADCL;
  wert_hi=ADCH;

  ret_wert=(wert_hi<<8)+wert_lo;

  if(ret_wert>1023) ret_wert=1023; // nur zur Sicherheit


  return ret_wert;
}

//-----------------------------------------------------------------------------
#if USE_MK3_ADC_READ_CH==1
unsigned int MK3_ADC_READ_CH(ADC_KAN kan) { // lesen eines bestimmten Kanals
  unsigned int ret_wert=0;

  // umschalten des Kanals
  MK3_ADC_CHANNEL(kan);

  // auslesen des Wertes
  ret_wert=MK3_ADC_READ();

  return ret_wert;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_ADC_MITTEL_CH==1
unsigned int MK3_ADC_MITTEL_CH(ADC_KAN kan,ADC_MITTEL mw) { // Mittelwerte
  unsigned int ret_wert=0;
  unsigned summe=0;
  unsigned char n,m;

  // umschalten des Kanals
  MK3_ADC_CHANNEL(kan);

  // anzahl der Messungen
  // Vorsicht !! nicht mehr als 64 Messungen
  // sonst ueberlauf
  m=1;  
  if(mw==MW_1) m=1;
  if(mw==MW_2) m=2;
  if(mw==MW_4) m=4;
  if(mw==MW_8) m=8;


  // Loop der Messungen
  summe=0;
  for(n=0;n<m;n++) {
    // auslesen eines Wertes
    // und aufsummieren
    summe=summe+MK3_ADC_READ();
  }

  // Mittelwert bilden
  if(m==1) ret_wert=summe;
  if(m==2) ret_wert=(summe >> 1); // dividieren durch 2
  if(m==4) ret_wert=(summe >> 2); // dividieren durch 4
  if(m==8) ret_wert=(summe >> 3); // dividieren durch 8

  return ret_wert;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_ADC_REF==1
void MK3_ADC_SET_REF(ADC_REF uref) { // umschalten der Referenz-Spannung
  unsigned char wert,neu;

  // alten Wert auslesen
  wert=(ADMUX&0x3F);
  // neuen Wert ermitteln
  neu=0x40;
  if(uref==REF_AREF) neu=0x00;
  if(uref==REF_AVCC) neu=0x40; // default
  if(uref==REF_INT) neu=0xC0;

  // neuen Wert setzen
  ADMUX=(wert|neu);
}
#endif

